from typing import List

from jinja2 import Environment, PackageLoader, select_autoescape
from pydantic import BaseModel


class Company(BaseModel):
    vatNumber: str
    country: str
    branch: int


class Address(BaseModel):
    postalCode: int
    city: str


class Issuer(Company):
    pass


class CounterPart(Company):
    address: Address


class InvoiceHeader(BaseModel):
    series: str
    aa: int
    issueDate: str
    invoiceType: str
    currency: str


class PaymentMethodDetails(BaseModel):
    type: int
    amount: float
    paymentMethodInfo: str


class IncomeClassification(BaseModel):
    type: str
    category: str
    amount: float


class InvoiceDetails(BaseModel):
    lineNumber: int
    netValue: float
    vatAmount: float
    vatCategory: str
    discountOption: bool
    classification: IncomeClassification


class Taxes(BaseModel):
    type: int
    category: int
    underlyingValue: float
    amount: float


class InvoiceSummary(BaseModel):
    net: float = 0.0
    vat: float = 0.0
    withheld: float = 0.0
    fees: float = 0.0
    stamp_duty: float = 0.0
    other_taxes: float = 0.0
    deductions: float = 0.0
    gross: float = 0.0
    classifications: List[IncomeClassification]


class Invoice(BaseModel):
    issuer: Issuer
    counterpart: CounterPart
    header: InvoiceHeader
    payment: PaymentMethodDetails
    # invoice details
    details: List[InvoiceDetails]
    taxes: List[Taxes] = []
    summary: InvoiceSummary

    def to_xml(self):
        """
        Renders the XML template
        """
        engine = Environment(
            loader=PackageLoader('mydatapy', 'templates'),
            autoescape=['xml']
        )
        template = engine.get_template("invoice.xml")
        return template.render(invoice=self)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mydatapy', 'mydatapy.models', 'mydatapy.templates']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=2.11.2,<3.0.0',
 'pydantic>=1.7.3,<2.0.0',
 'requests-xml>=0.2.3,<0.3.0',
 'requests>=2.25.1,<3.0.0',
 'xmltodict>=0.12.0,<0.13.0']

setup_kwargs = {
    'name': 'mydatapy',
    'version': '0.1.1',
    'description': 'A simple wrapper of mydata platform of greek tax registry',
    'long_description': None,
    'author': 'Dimitrios Strantsalis',
    'author_email': 'dstrants@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Optional, Union

import requests
import xmltodict


class MyDataException(Exception):
    pass


class XmlValidationError(MyDataException):
    def __init__(self, errors: list) -> None:
        self.errors: list = errors

    def __str__(self) -> str:
        return str(self.errors)


class MyDataClient:
    """
    MyData API Wrapper
    The class wraps the mydata API to send and receive docs
    from the platform

    Params:
    -------------------------------------------------------
    user_id: str -> The id of the user on mydata.
    api_key: str -> The Ocp API Subscription key.
    base_url: str -> The base url of the mydata api. (defaults to dev)
    """
    def __init__(self, user_id: str, api_key: str, base_url: Optional[str] = None) -> None:
        self.user_id: str = user_id
        self.api_key: str = api_key
        self.base_url: str = base_url or "https://mydata-dev.azure-api.net"

    def call(self, endpoint: str, method: str = "GET", body: Optional[Union[dict, str]] = None,
             params: Optional[dict] = None) -> dict:
        """
        Abstract method to perform alls calls to my data api.
        """
        headers = {
            'aade-user-id': self.user_id,
            'Ocp-Apim-Subscription-Key': self.api_key
        }

        if isinstance(body, str):
            body = body.encode('utf-8')

        response = requests.request(
            method=method, url=f"{self.base_url}/{endpoint}", params=params, headers=headers,
            data=body
        )
        if not response.ok:
            raise ValueError(response.text)
        body = xmltodict.parse(response.text)
        if 'errors' in (result := body['ResponseDoc']['response']):
            raise XmlValidationError(result['errors'])
        return result

    def send_invoice(self, invoice: str):
        return self.call(
            method="POST", body=invoice, endpoint="SendInvoices"
        )
